#include "MediumMagboltz.hh"
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <fstream>

int main(int argc, const char **argv)
{
  using namespace std;
  //--
  // Check the input arguments.
  // --
  if (argc != 5) {
    cerr << "usage: " << endl
         << "  ./gas1 <run> <E> <p> <t>" << endl;
    abort();
  }

  //--
  // Reference point.
  // --
  int    Runno0 = 19987;   // Run Number
  double P0     = 1014.44; // [hPa]
  double T0     = 291.16;  // [K]
  double E0     = 230.;    // [K]

  //--
  // Read in input arguments.
  // --
  int    Runno  = Runno0;  // Run Number
  double P      = P0;      // [hPa]
  double T      = T0;      // [K]
  double E      = E0;      // [V/cm]
  //double E      = 35500.;   // [V/cm] for 1st amp GEM  (0.1mm)
  //double E      = 900.;     // [V/cm] for transfer gap  (4.0mm)
  //double E      = 31500.;   // [V/cm] for 2nd amp GEM   (0.1mm)
  //double E      = 2700.;    // [V/cm] for induction gap (2.0mm)
  double B      =   1.;    // [T]
  double A      =   0.;    // [rad]

  stringstream par(argv[1]);
    par >> Runno;
    par.clear();
    par.str(argv[2]);
    par >> E;
    par.clear();
    par.str(argv[3]);
    par >> P;
    par.clear();
    par.str(argv[4]);
    par >> T;
  cerr << "E=" << E << "[V/cm]" << endl;

  //--
  // Set output file names.
  //--
  stringstream ofname;
  //ofname << "r" << Runno << ".dat" << ends;
  ofname << "E" << E << ".dat" << ends;

  ofstream ofile(ofname.str().data());

  cerr << " Run = " << Runno 
       << " E = " << E << " [V/cm]" 
       << " P = " << P << "[hPa]"
       << " T = " << T << "[K]" << endl; 
  ofile << Runno << " " << E << " " << P << " " << T << " "; 
 
  ofname.clear();
  ofname.str("");
  ofname << "r" << Runno << ".E" << E << ".gas" << ends;

  //--
  // Create a Medium object using Magboltz.
  //--
  Garfield::MediumMagboltz *gas = new Garfield::MediumMagboltz();

#if 1
  //--
  // Create a gas file.
  //--
  gas->SetComposition("ar", 95., "c4h10", 2., "cf4", 3.);
  gas->SetPressure(P0/1013. * 760.);  
  gas->SetTemperature(T0); 
  gas->SetFieldGrid(E, E, 1, false, // (emin, emax, ne, logE,
		    B, B, 1,        //  bmin, bmax, nb)
		    A, A, 1);       // amin, amax, na)

  gas->EnableDebugging();
  //gas->GenerateGasTable(400);
  //gas->GenerateGasTable(1000);
  gas->GenerateGasTable(10); 
 
  gas->WriteGasFile(ofname.str().data());
#else
  //--
  // Read in a gas file.
  //--
  gas->LoadGasFile(ofname.str().data()); 
#endif

  //--
  // Calculate diffusion coefficients.
  //--
  double Dt = -9999.;
  double Dl = -9999.;
  double ans1, ans2;
  if (gas->ElectronDiffusion(0.,0.,E,0.,0.,B,ans1,ans2)) {
    double corr = sqrt((T/P)/(T0/P0));
    Dt = ans2*corr;
    Dl = ans1*corr;
  }
  cerr << "Diffusion Coefficients at"
       << " T = " << T-273. << "[C] and" 
       << " P = " << P      << "[hPa]" <<  endl;
  cerr << "    Dt = " << Dt*1.e4 << "[um/sqrt(cm)]" << endl
       << "    Dl = " << Dl*1.e4 << "[um/sqrt(cm}]" << endl;
  ofile << Dt << " " << Dl << endl;

  return 0;
}
